import unittest
from helpers import extract_domain


class TestDomainExtraction(unittest.TestCase):
    """Ensure the extraction of domain works as expected."""

    def test_extract_domain_endpoint(self):
        url = "https://www.inoopa.com/contact"
        domain = extract_domain(url)
        self.assertEqual(domain, "inoopa.com")

    def test_extract_domain_endpoint_with_params(self):
        url = "https://www.inoopa.com/contact/index.php?param=value"
        domain = extract_domain(url)
        self.assertEqual(domain, "inoopa.com")

    def test_extract_domain_endpoint_with_subdomain(self):
        url = "https://www.inoopa.com/contact"
        domain = extract_domain(url)
        self.assertEqual(domain, "inoopa.com")

        url = "https://test.www.inoopa.com/contact"
        domain = extract_domain(url)
        self.assertEqual(domain, "inoopa.com")

    def test_extract_domain_with_double_extension(self):
        url = "https://www.inoopa.co.uk/contact.html"
        domain = extract_domain(url)
        self.assertEqual(domain, "inoopa.co.uk")


if __name__ == "__main__":
    unittest.main()



from typing import Literal
from datetime import datetime
from dataclasses import dataclass, field

from inoopa_utils.typesense_helpers import datetime_to_unix_timestamp


@dataclass(slots=True)
class NaceCodeTypesense:
    country: str
    level: int
    code: str
    section_label: str
    section_code: str
    label_en: str
    label_en_extended: str
    label_en_embedding: list[float] | None = None
    label_en_extended_embedding: list[float] | None = None
    label_fr: str | None = None
    label_fr_extended: str | None = None
    label_fr_extended_embedding: list[float] | None = None
    label_fr_embedding: list[float] | None = None
    label_nl: str | None = None
    label_nl_extended: str | None = None
    label_nl_extended_embedding: list[float] | None = None
    label_nl_embedding: list[float] | None = None


@dataclass
class EntityName:
    name: str | None = None
    name_fr: str | None = None
    name_nl: str | None = None
    name_de: str | None = None
    website: str | None = None


@dataclass(init=False)
class CompanyNameTypesense(EntityName):
    _id: str
    establishments: list[EntityName] | None = None

    # We have to manually define the __init__ method here because of the way dataclasses work with inheritance
    def __init__(self, _id: str, establishments: list[EntityName] | None = None, **kwargs):
        super().__init__(**kwargs)
        self._id = _id
        self.establishments = establishments


@dataclass(slots=True)
class WebsitePageTypesense:
    """New implementation of the website page Typesense collection."""

    page_url: str
    base_url: str
    page_type: Literal["home", "about", "contact"]
    text: str | None = None
    embedding: list[float] | None = None
    last_crawling: int = field(default_factory=lambda: datetime_to_unix_timestamp(datetime.now()))


@dataclass(slots=True)
class WebsiteTypesense:
    companies_id: list[str] | None = None
    # In typesense we can't search for a null value, we set has_companies_id to True/False to be able to search for it
    # See: https://typesense.org/docs/guide/tips-for-searching-common-types-of-data.html#searching-for-null-or-empty-values
    has_companies_id: bool = True
    # Typesense doesn't support datetime objects. So we convert it to unix timestamp.
    # See: https://typesense.org/docs/26.0/api/collections.html#notes-on-indexing-common-types-of-data
    last_crawling: int = field(default_factory=lambda: datetime_to_unix_timestamp(datetime.now()))
    mongo_best_website_url: str | None = None
    home_page_url: str | None = None
    home_page_status_code: int | None = None
    home_page_text: str | None = None
    home_page_embedding: list[float] | None = None
    about_page_url: str | None = None
    about_page_status_code: int | None = None
    about_page_text: str | None = None
    about_page_embedding: list[float] | None = None
    contact_page_url: str | None = None
    contact_page_status_code: int | None = None
    contact_page_text: str | None = None
    contact_page_embedding: list[float] | None = None


@dataclass(slots=True)
class WebsiteTypesenseV2:
    """New implementation of the website Typesense collection."""

    companies_id: list[str]
    # Typesense doesn't support datetime objects. So we convert it to unix timestamp.
    # See: https://typesense.org/docs/26.0/api/collections.html#notes-on-indexing-common-types-of-data
    mongo_best_website_url: str
    domain: str
    last_crawling: int = field(default_factory=lambda: datetime_to_unix_timestamp(datetime.now()))
    page_about_url: str | None = None
    page_about_text: str | None = None
    page_about_embedding: str | None = None
    page_contact_url: str | None = None
    page_contact_text: str | None = None
    page_contact_embedding: str | None = None



from datetime import datetime
from dataclasses import dataclass


@dataclass(slots=True)
class CO2DataLegacy:
    """
    Dataclass to store the co2 data from our legacy SQL database table (co2_emissions).

    Each object represents a single company.
    These data are not updated anymore. We stopped development on this project.
    This is kept for reference purpose only.
    """
    # Company ID (same _id as in company collection with leading zeros BE:012345667)
    _id: str
    # Company ID (id in the old format (VAT without leading zeros) BE:12345667)
    legacy_entity_id: str
    date: datetime
    # ISO code of the country (BE, FR, NL)
    country: str
    first_best_nace_4: str | None = None
    first_best_nace_score: float | None = None
    first_best_nace_count: int | None = None
    second_best_nace_4: str | None = None
    second_best_nace_score: float | None = None
    third_best_nace_4: str | None = None
    third_best_nace_score: float | None = None
    first_declared_nace_4: str | None = None
    first_declared_nace_count: int | None = None
    estimated_staff: int | None = None
    employee_category: int | None = None
    staff_type: str | None = None
    average_ratio_scope1_first_best_nace: float | int | None = None
    average_ratio_scope1_second_best_nace: float | int | None = None
    average_ratio_scope1_third_best_nace: float | int | None = None
    average_ratio_scope1_declared_nace: float | int | None = None
    scope1_weight: float | int | None = None
    IC_80_firstnace_scope1: float | int | None = None
    scope1_declared: float | int | None = None
    average_ratio_scope2_first_best_nace: float | int | None = None
    average_ratio_scope2_second_best_nace: float | int | None = None
    average_ratio_scope2_third_best_nace: float | int | None = None
    average_ratio_scope2_declared_nace: float | int | None = None
    scope2_weight: float | int | None = None
    IC_80_firstnace_scope2: float | int | None = None
    scope2_declared: float | int | None = None
    last_updated: datetime | None = None



import sys
import random
import requests
from typing import Literal
from tldextract import extract
from functools import lru_cache
from inoopa_utils.inoopa_logging import create_logger
from urllib.parse import urlparse


def extract_domain(url: str) -> str:
    """
    Extract the domain from a url. We dedicate a function here to make sure we do it the same way everywhere.

    ex: https://www.inoopa.com/contact -> inoopa.com
    """
    # if http is not present, we can't parse the domain
    if not url.startswith("https://") and not url.startswith("http://"):
        url = f"https://{url}"
    return extract(url).registered_domain


def extract_base_url(url: str) -> str:
    """Extract the base url from a url. Ex: https://www.inoopa.com/contact -> https://www.inoopa.com"""
    if not url.startswith("https://") and not url.startswith("http://"):
        url = f"https://{url}"
    parsed_url = urlparse(url)
    return f"{parsed_url.scheme}://{parsed_url.netloc}"


@lru_cache()
def get_all_latest_user_agents() -> list[str]:
    """
    Daily updated list of user agents

    :return: A list of user agents.
    """
    url = "https://jnrbsn.github.io/user-agents/user-agents.json"
    r = requests.get(url)
    r.raise_for_status()
    return r.json()


def get_random_user_agent() -> str:
    """Get a random user agent. the user agents list is cached."""
    return random.choice(get_all_latest_user_agents())


def get_latest_user_agent(
    operating_system: Literal["macintosh", "windows", "x11"] = "macintosh",
    browser: Literal["version", "chrome", "firefox"] = "version",
) -> str:
    """
    General function to fetch the latest user agent for a given operating system and browser.

    :param operating_system: The operating system to search for. Default is macintosh, X11 is Linux.
    :param browser: The browser to search for. Version is the latest Safari.
    :return: The latest user agent for the given operating system and browser.
    """
    user_agents = get_all_latest_user_agents()
    for user_agent in user_agents:
        user_agent_lower = user_agent.lower()
        if operating_system.lower() in user_agent_lower and browser.lower() in user_agent_lower:
            return user_agent
    raise ValueError(f"No user-agent found for OS: {operating_system} and browser: {browser}")


def DEBUG_get_variable_memory_usage(name: str, limit: int = 5) -> None:
    """
    Log the memory usage of the 5 most memory-consuming variables. Usefull to identify memory leaks.

    :pram name: The name of the logger to use.
    :param limit: The number of variables to print. (Only log the top X)
    """
    logger = create_logger(f"INOOPA.MEMORY_USAGE.{name.upper()}", logging_level="DEBUG")
    # Access local and global variables
    variables = {**globals(), **locals()}  # Combine both scopes

    # Calculate sizes of all variables
    variable_sizes = []
    for var_name, var_value in variables.items():
        try:
            size = sys.getsizeof(var_value)
            variable_sizes.append((var_name, size))
        except Exception:
            variable_sizes.append((var_name, 0))

    # Sort variables by size and get the top ones
    top_variables = sorted(variable_sizes, key=lambda x: x[1], reverse=True)[:limit]

    # Print the results in KB
    logger.debug(f"{'Variable Name':<30} {'Size (KB)':>15}")
    logger.debug("-" * 50)
    for var_name, size in top_variables:
        size_kb = size / 1024  # Convert bytes to KB
        logger.debug(f"{var_name:<30} {size_kb:>15.2f}")



from dataclasses import dataclass, field
from datetime import datetime


@dataclass(slots=True)
class WebsiteStructure:
    """Dataclass to represent the website_structure MongoDB collection's documents."""

    _id: str  # This is the base_url. We use it as the unique key
    domain: str

    # Extracted from the website's structure
    home_url: str | None = None
    about_url: str | None = None
    contact_url: str | None = None

    # Extracted from the website's content
    company_mission: str | None = None
    phones: list[str] = field(default_factory=list)
    emails: list[str] = field(default_factory=list)
    social_networks: list[str] = field(default_factory=list)
    vat_numbers: list[str] = field(default_factory=list)

    # Updated by the best_finder
    companies_ids: list[str] = field(default_factory=list)
    # Updated by the websites_crawler
    last_crawling: datetime = field(default_factory=datetime.now)
